@@ -22,6 +22,7 @@
 
 #include <goffice/utils/go-glib-extras.h>
 #include <goffice/utils/go-libxml-extras.h>
+#include <goffice/utils/go-file.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
@@ -35,7 +36,9 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
 #include <locale.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -637,6 +640,7 @@ go_plugin_read_service_list (GOPlugin *plugin, xmlNode *tree, ErrorInfo **ret_er
 
 	g_return_val_if_fail (tree != NULL, NULL);
 
+	GO_INIT_RET_ERROR_INFO (ret_error);
 	node = e_xml_get_child_by_name (tree, (xmlChar *)"services");
 	if (node == NULL)
 		return NULL;
@@ -732,7 +736,8 @@ go_plugin_read (GOPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_error)
 	file_name = g_build_filename (dir_name, PLUGIN_INFO_FILE_NAME, NULL);
 	doc = go_xml_parse_file (file_name);
 	if (doc == NULL || doc->xmlRootNode == NULL || strcmp (doc->xmlRootNode->name, "plugin") != 0) {
-		if (access (file_name, R_OK) != 0) {
+		char *uri = go_filename_to_uri (file_name);
+		if (go_file_access (uri, R_OK) != 0) {
 			*ret_error = error_info_new_printf (
 			             _("Can't read plugin info file (\"%s\")."),
 			             file_name);
@@ -742,6 +747,7 @@ go_plugin_read (GOPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_error)
 			             file_name);
 		}
 		g_free (file_name);
+		g_free (uri);
 		xmlFreeDoc (doc);
 		return;
 	}
@@ -866,7 +872,7 @@ static void
 plugin_get_loader_if_needed (GOPlugin *plugin, ErrorInfo **ret_error)
 {
 	GType loader_type;
-	ErrorInfo *error;
+	ErrorInfo *error = NULL;
 
 	g_return_if_fail (IS_GO_PLUGIN (plugin));
 
@@ -1038,6 +1044,8 @@ go_plugin_deactivate (GOPlugin *plugin, ErrorInfo **ret_error)
 			go_plugin_use_unref (plugin_dependency_get_plugin (dep));
 		);
 		if (plugin->loader != NULL) {
+			if (go_plugin_loader_is_base_loaded (plugin->loader))
+				go_plugin_loader_unload_base (plugin->loader, ret_error);
 			g_object_unref (plugin->loader);
 			plugin->loader = NULL;
 		}
@@ -1336,7 +1344,7 @@ go_plugin_list_read_for_subdirs_of_dir (const gchar *dir_name, ErrorInfo **ret_e
 
 	while ((d_name = g_dir_read_name (dir)) != NULL) {
 		gchar *full_entry_name;
-		ErrorInfo *error;
+		ErrorInfo *error = NULL;
 		GOPlugin *plugin;
 
 		if (strcmp (d_name, ".") == 0 || strcmp (d_name, "..") == 0)
@@ -1373,7 +1381,7 @@ go_plugin_list_read_for_subdirs_of_dir_list (GSList *dir_list, ErrorInfo **ret_e
 	GO_INIT_RET_ERROR_INFO (ret_error);
 	for (dir_iterator = dir_list; dir_iterator != NULL; dir_iterator = dir_iterator->next) {
 		gchar *dir_name;
-		ErrorInfo *error;
+		ErrorInfo *error = NULL;
 		GSList *dir_plugin_info_list;
 
 		dir_name = (gchar *) dir_iterator->data;
@@ -1522,8 +1530,9 @@ go_plugins_get_plugin_by_id (const gchar *plugin_id)
 }
 
 /**
- * plugin_db_mark_plugin_for_deactivation:
- * ...
+ * plugin_db_mark_plugin_for_deactivation :
+ * @plugin:
+ * @mark:
  */
 void
 go_plugin_db_mark_plugin_for_deactivation (GOPlugin *plugin, gboolean mark)
@@ -1543,8 +1552,8 @@ go_plugin_db_mark_plugin_for_deactivation (GOPlugin *plugin, gboolean mark)
 }
 
 /**
- * plugin_db_is_plugin_marked_for_deactivation:
- * ...
+ * plugin_db_is_plugin_marked_for_deactivation :
+ * @plugin: 
  */
 gboolean
 go_plugin_db_is_plugin_marked_for_deactivation (GOPlugin *plugin)
@@ -1768,6 +1777,84 @@ go_plugins_init (GOCmdContext *context,
 	}
 }
 
+/**
+ * go_plugins_add:
+ * @context     : #GOCmdContext used to report errors
+ *
+ * Adds new plugins to currently used plugins.
+ */
+void
+go_plugins_add (GOCmdContext *context,
+		 GSList const *known_states,
+		 GSList const *active_plugins,
+		 GSList *plugin_dirs,
+		 GType  default_loader_type)
+{
+	GSList *error_list = NULL;
+	ErrorInfo *error;
+	GSList *plugin_list;
+
+	plugin_dirs = g_slist_append (plugin_dirs, go_plugins_get_plugin_dir ());
+
+	go_default_loader_type = default_loader_type;
+	go_plugins_set_dirs (plugin_dirs);
+
+	GO_SLIST_FOREACH (known_states, char, state_str,
+		PluginFileState *state;
+
+		state = plugin_file_state_from_string (state_str);
+		if (state != NULL)
+			g_hash_table_insert (plugin_file_state_dir_hash, state->dir_name, state);
+	);
+	plugin_file_state_hash_changed = FALSE;
+
+	/* collect information about the available plugins */
+	g_slist_free (available_plugins);
+	available_plugins = go_plugin_list_read_for_all_dirs (&error);
+	GO_SLIST_FOREACH (available_plugins, GOPlugin, plugin,
+		g_hash_table_insert (
+			available_plugins_id_hash,
+			(gpointer) go_plugin_get_id (plugin), plugin);
+	);
+	if (error != NULL) {
+		GO_SLIST_PREPEND (error_list, error_info_new_str_with_details (
+			_("Errors while reading info about available plugins."), error));
+	}
+
+	/* get descriptors for all previously active plugins */
+	plugin_list = NULL;
+	GO_SLIST_FOREACH (active_plugins, char, plugin_id,
+		GOPlugin *plugin = go_plugins_get_plugin_by_id (plugin_id);
+		if (plugin != NULL)
+			GO_SLIST_PREPEND (plugin_list, plugin);
+	);
+
+	/* get descriptors for new plugins */
+	g_hash_table_foreach (
+		plugin_file_state_dir_hash,
+		(GHFunc) ghf_collect_new_plugins,
+		&plugin_list);
+
+	plugin_list = g_slist_reverse (plugin_list);
+	go_plugin_db_activate_plugin_list (plugin_list, &error);
+	g_slist_free (plugin_list);
+	if (error != NULL) {
+		GO_SLIST_PREPEND (error_list, error_info_new_str_with_details (
+			_("Errors while activating plugins."), error));
+	}
+
+	/* report initialization errors */
+	if (error_list != NULL) {
+		GO_SLIST_REVERSE (error_list);
+		error = error_info_new_str_with_details_list (
+		        _("Errors while initializing plugin system."),
+		        error_list);
+
+		go_cmd_context_error_info (context, error);
+		error_info_free (error);
+	}
+}
+
 static void
 ghf_collect_used_plugin_state_strings (gpointer key, gpointer value, gpointer user_data)
 {
